using System;
using System.Collections.Generic;
using System.IO;
using System.Text;

namespace AsusTrxExtract
{
    sealed class ImageFile
    {
        #region Members

        // some predefined header values
        private const string BRCM = "brcm";
        private const int BRCM_HEADER_LENGHT = 44;
        private const string TRX_MAGIC = "hdr0";
        private const int TYPE_TAG = 21;
        private const int TYPE_FLASH = 18;
        private const int TYPE_DISK = 19;

        // image data
        private string buildInfo = string.Empty;
        private string[] buildInfoArray = null;
        private byte[] flashFile = null;
        private byte[] diskFile = null;

        #endregion

        #region Constructors

        public ImageFile(byte[] data)
        {
            int dataLen = 0;
            Encoding e = Encoding.ASCII;

            // check for correct magic
            if (e.GetString(data, 0, 4).ToLower() != BRCM)
                throw new ArgumentException("Invalid image header");
            // check number of sections - must be 3
            if (BitConverter.ToInt32(data, 4) != 3)
                throw new ArgumentException("Incorrect number of image sections");
            // check section types
            // 1. TYPE_TAG
            // 2. TYPE_FLASH
            // 3. TYPE_DISK
            if (BitConverter.ToInt32(data, 8) != TYPE_TAG)
                throw new ArgumentException("Incorrect section type for first section");
            dataLen = BitConverter.ToInt32(data, 12);
            buildInfo = e.GetString(data, BRCM_HEADER_LENGHT, dataLen - 1);
            buildInfoArray = buildInfo.Split('\n');
            if (BitConverter.ToInt32(data, 20) != TYPE_FLASH)
                throw new ArgumentException("Incorrect section type for second section");
            dataLen = BitConverter.ToInt32(data, 24);
            flashFile = new byte[dataLen];
            Buffer.BlockCopy(data, BRCM_HEADER_LENGHT + BuildInfo.Length + 1, flashFile, 0, dataLen);
            if (BitConverter.ToInt32(data, 32) != TYPE_DISK)
                throw new ArgumentException("Incorrect section type for third section");
            dataLen = BitConverter.ToInt32(data, 36);
            diskFile = new byte[dataLen];
            Buffer.BlockCopy(data, BRCM_HEADER_LENGHT + BuildInfo.Length + flashFile.Length + 1, diskFile, 0, dataLen);
        }

        #endregion

        #region Properties

        public string BuildInfo
        {
            get { return buildInfo; }
        }

        public string Name
        {
            get
            {
                if (buildInfoArray != null && buildInfoArray.Length > 0)
                {
                    return buildInfoArray[0];
                }
                else
                {
                    return string.Empty;
                }
            }
        }

        public string Version
        {
            get
            {
                foreach (string str in buildInfoArray)
                {
                    if (str.ToLower().StartsWith("version:"))
                        return str.Substring(9);
                }

                return string.Empty;
            }
        }

        public string BuildDate
        {
            get
            {
                foreach (string str in buildInfoArray)
                {
                    if (str.ToLower().StartsWith("build date:"))
                        return str.Substring(11);
                }

                return string.Empty;
            }
        }

        public string BuildBy
        {
            get
            {
                foreach (string str in buildInfoArray)
                {
                    if (str.ToLower().StartsWith("build by:"))
                        return str.Substring(10);
                }

                return string.Empty;
            }
        }

        public int FlashImageSize
        {
            get { return flashFile.Length; }
        }

        public int DiskImageSize
        {
            get { return diskFile.Length; }
        }

        #endregion

        #region Methods

        public void writeImageFilesToDisk(string baseName)
        {
            FileStream fs = File.OpenWrite(Path.GetFileNameWithoutExtension(baseName) + "_flash.trx");
            fs.Write(flashFile, 0, flashFile.Length);
            fs.Close();
            fs.Dispose();
            fs = File.OpenWrite(Path.GetFileNameWithoutExtension(baseName) + "_disk.trx");
            fs.Write(diskFile, 0, diskFile.Length);
            fs.Close();
            fs.Dispose();
        }

        #endregion
    }
}
